@@ -2,11 +2,7 @@
 # MIT License (see LICENSE or https://opensource.org/licenses/MIT)
 
 import time
-
-from collections import (
-    namedtuple,
-    OrderedDict,
-)
+from collections import OrderedDict, namedtuple
 
 from smbprotocol.structure import (
     BytesField,
@@ -18,8 +14,7 @@ from smbprotocol.structure import (
     TextField,
 )
 
-
-DFSTarget = namedtuple('DFSTarget', ['target_path', 'set_boundary'])
+DFSTarget = namedtuple("DFSTarget", ["target_path", "set_boundary"])
 """A TargetEntry element in the TargetList ReferralCache.
 
 Contains the DFS target details for a referral request. This object is defined in [MS-DFSC] under
@@ -54,12 +49,12 @@ class DomainEntry:
 
     @property
     def domain_name(self):  # type: () -> str
-        """ The domain DFS path. """
+        """The domain DFS path."""
         return self._referral.dfs_path
 
     @property
     def dc_hint(self):  # type: () -> str
-        """ The last known good domain hostname in domain_list. """
+        """The last known good domain hostname in domain_list."""
         return self.domain_list[self._domain_hint_idx]
 
     @dc_hint.setter
@@ -74,19 +69,19 @@ class DomainEntry:
 
     @property
     def is_expired(self):  # type: () -> bool
-        """ Whether the hint has expired or not. """
-        return ((time.time() - self._start_time) - self._referral['time_to_live'].get_value()) >= 0
+        """Whether the hint has expired or not."""
+        return ((time.time() - self._start_time) - self._referral["time_to_live"].get_value()) >= 0
 
     @property
     def is_valid(self):  # type: () -> bool
-        """ Whether the domain entry has had a DC referral response or not. """
+        """Whether the domain entry has had a DC referral response or not."""
         return self._domain_hint_idx is not None and not self.is_expired
 
     def process_dc_referral(self, referral):  # type: (DFSReferralResponse) -> None
         if self._domain_hint_idx is None:
             self._domain_hint_idx = 0
 
-        for dc_entry in referral['referral_entries'].get_value():
+        for dc_entry in referral["referral_entries"].get_value():
             for dc_hostname in dc_entry.network_address:
                 if dc_hostname not in self.domain_list:
                     self.domain_list.append(dc_hostname)
@@ -102,8 +97,8 @@ class ReferralEntry:
     """
 
     def __init__(self, referral):  # type: (DFSReferralResponse) -> None
-        referrals = referral['referral_entries'].get_value()
-        self._referral_header_flags = referral['referral_header_flags']
+        referrals = referral["referral_entries"].get_value()
+        self._referral_header_flags = referral["referral_header_flags"]
         self._referrals = referrals  # type: List[Union[DFSReferralEntryV1, DFSReferralEntryV2, DFSReferralEntryV3]]
         self._start_time = time.time()  # type: float
         self._target_hint_idx = 0  # type: int
@@ -114,7 +109,7 @@ class ReferralEntry:
 
     @property
     def is_root(self):  # type: () -> bool
-        return self._referrals[self._target_hint_idx]['server_type'].has_flag(DFSServerTypes.ROOT_TARGETS)
+        return self._referrals[self._target_hint_idx]["server_type"].has_flag(DFSServerTypes.ROOT_TARGETS)
 
     @property
     def is_link(self):  # type: () -> bool
@@ -127,7 +122,7 @@ class ReferralEntry:
     @property
     def is_expired(self):  # type: () -> bool
         referral = self._referrals[self._target_hint_idx]
-        return ((time.time() - self._start_time) - referral['time_to_live'].get_value()) >= 0
+        return ((time.time() - self._start_time) - referral["time_to_live"].get_value()) >= 0
 
     @property
     def target_failback(self):  # type: () -> bool
@@ -150,13 +145,15 @@ class ReferralEntry:
     @property
     def target_list(self):  # type: () -> List[DFSTarget]
         return [
-            DFSTarget(target_path=e.network_address,
-                      set_boundary=e['referral_entry_flags'].has_flag(DFSReferralEntryFlags.TARGET_SET_BOUNDARY))
+            DFSTarget(
+                target_path=e.network_address,
+                set_boundary=e["referral_entry_flags"].has_flag(DFSReferralEntryFlags.TARGET_SET_BOUNDARY),
+            )
             for e in self._referrals
         ]
 
     def __iter__(self):  # type: () -> Iterator[DFSTarget]
-        """ Iterates through the target_list with a priority being the hinted value. """
+        """Iterates through the target_list with a priority being the hinted value."""
         yield self.target_list[self._target_hint_idx]
 
         for idx, target in enumerate(self.target_list):
@@ -172,6 +169,7 @@ class DFSReferralRequestFlags:
 
     RequestFlags
     """
+
     SITE_NAME = 0x00000001
 
 
@@ -181,6 +179,7 @@ class DFSReferralHeaderFlags:
 
     ReferralHeaderFlags
     """
+
     REFERRAL_SERVERS = 0x00000001
     STORAGE_SERVERS = 0x00000002
     TARGET_FAIL_BACK = 0x00000004
@@ -192,6 +191,7 @@ class DFSServerTypes:
 
     ServerType
     """
+
     NON_ROOT_TARGETS = 0x0000
     ROOT_TARGETS = 0x0001
 
@@ -202,6 +202,7 @@ class DFSReferralEntryFlags:
 
     ReferralEntryFlags
     """
+
     NAME_LIST_REFERRAL = 0x0002
     TARGET_SET_BOUNDARY = 0x0004
 
@@ -214,10 +215,12 @@ class DFSReferralRequest(Structure):
     """
 
     def __init__(self):
-        self.fields = OrderedDict([
-            ('max_referral_level', IntField(size=2, default=4)),
-            ('request_file_name', TextField(null_terminated=True)),
-        ])
+        self.fields = OrderedDict(
+            [
+                ("max_referral_level", IntField(size=2, default=4)),
+                ("request_file_name", TextField(null_terminated=True)),
+            ]
+        )
         super(DFSReferralRequest, self).__init__()
 
 
@@ -229,30 +232,49 @@ class DFSReferralRequestEx(Structure):
     """
 
     def __init__(self):
-        self.fields = OrderedDict([
-            ('max_referral_level', IntField(size=2, default=4)),
-            ('request_flags', FlagField(size=2, flag_type=DFSReferralRequestFlags)),
-            ('request_data_length', IntField(
-                size=4,
-                default=lambda s: 4 + s['request_file_name_length'].get_value() + s['site_name_length'].get_value(),
-            )),
-            ('request_file_name_length', IntField(
-                size=2,
-                default=lambda s: len(s['request_file_name']),
-            )),
-            ('request_file_name', TextField(
-                null_terminated=True,
-                size=lambda s: s['request_file_name_length'].get_value(),
-            )),
-            ('site_name_length', IntField(
-                size=2,
-                default=lambda s: len(s['site_name']),
-            )),
-            ('site_name', TextField(
-                null_terminated=True,
-                size=lambda s: s['site_name_length'].get_value(),
-            )),
-        ])
+        self.fields = OrderedDict(
+            [
+                ("max_referral_level", IntField(size=2, default=4)),
+                ("request_flags", FlagField(size=2, flag_type=DFSReferralRequestFlags)),
+                (
+                    "request_data_length",
+                    IntField(
+                        size=4,
+                        default=lambda s: 4
+                        + s["request_file_name_length"].get_value()
+                        + s["site_name_length"].get_value(),
+                    ),
+                ),
+                (
+                    "request_file_name_length",
+                    IntField(
+                        size=2,
+                        default=lambda s: len(s["request_file_name"]),
+                    ),
+                ),
+                (
+                    "request_file_name",
+                    TextField(
+                        null_terminated=True,
+                        size=lambda s: s["request_file_name_length"].get_value(),
+                    ),
+                ),
+                (
+                    "site_name_length",
+                    IntField(
+                        size=2,
+                        default=lambda s: len(s["site_name"]),
+                    ),
+                ),
+                (
+                    "site_name",
+                    TextField(
+                        null_terminated=True,
+                        size=lambda s: s["site_name_length"].get_value(),
+                    ),
+                ),
+            ]
+        )
         super(DFSReferralRequestEx, self).__init__()
 
 
@@ -264,21 +286,26 @@ class DFSReferralResponse(Structure):
     """
 
     def __init__(self):
-        self.fields = OrderedDict([
-            ('path_consumed', IntField(size=2)),
-            ('number_of_referrals', IntField(size=2)),
-            ('referral_header_flags', FlagField(size=4, flag_type=DFSReferralHeaderFlags)),
-            ('referral_entries', ListField(
-                list_count=lambda s: s['number_of_referrals'].get_value(),
-                unpack_func=lambda s, b: self._create_dfs_referral_entry(b)
-            )),
-            ('string_buffer', BytesField()),
-        ])
+        self.fields = OrderedDict(
+            [
+                ("path_consumed", IntField(size=2)),
+                ("number_of_referrals", IntField(size=2)),
+                ("referral_header_flags", FlagField(size=4, flag_type=DFSReferralHeaderFlags)),
+                (
+                    "referral_entries",
+                    ListField(
+                        list_count=lambda s: s["number_of_referrals"].get_value(),
+                        unpack_func=lambda s, b: self._create_dfs_referral_entry(b),
+                    ),
+                ),
+                ("string_buffer", BytesField()),
+            ]
+        )
         super(DFSReferralResponse, self).__init__()
 
     def _create_dfs_referral_entry(self, data):
         results = []
-        for _ in range(self.fields['number_of_referrals'].get_value()):
+        for _ in range(self.fields["number_of_referrals"].get_value()):
             b_version = data[:1]
             if b_version == b"\x01":
                 referral_entry = DFSReferralEntryV1()
@@ -296,7 +323,7 @@ class DFSReferralResponse(Structure):
         # on the entry size(s) that come after that entry.
         entry_offset = 0
         for referral_entry in reversed(results):
-            entry_offset += referral_entry['size'].get_value()
+            entry_offset += referral_entry["size"].get_value()
             referral_entry.process_string_buffer(data, entry_offset)
 
         return results
@@ -310,18 +337,20 @@ class DFSReferralEntryV1(Structure):
     """
 
     def __init__(self):
-        self.fields = OrderedDict([
-            ('version_number', IntField(size=2, default=1)),
-            ('size', IntField(size=2)),
-            ('server_type', EnumField(size=2, enum_type=DFSServerTypes)),
-            ('referral_entry_flags', FlagField(size=2, flag_type=DFSReferralEntryFlags)),
-            ('share_name', TextField(null_terminated=True)),
-        ])
+        self.fields = OrderedDict(
+            [
+                ("version_number", IntField(size=2, default=1)),
+                ("size", IntField(size=2)),
+                ("server_type", EnumField(size=2, enum_type=DFSServerTypes)),
+                ("referral_entry_flags", FlagField(size=2, flag_type=DFSReferralEntryFlags)),
+                ("share_name", TextField(null_terminated=True)),
+            ]
+        )
         super(DFSReferralEntryV1, self).__init__()
 
     @property
     def network_address(self):
-        return self['share_name'].get_value()
+        return self["share_name"].get_value()
 
     def process_string_buffer(self, buffer, entry_offset):
         # The V1 entry does not use a string buffer so this is a no-op.
@@ -336,32 +365,34 @@ class DFSReferralEntryV2(Structure):
     """
 
     def __init__(self):
-        self.fields = OrderedDict([
-            ('version_number', IntField(size=2, default=2)),
-            ('size', IntField(size=2)),
-            ('server_type', FlagField(size=2, flag_type=DFSServerTypes)),
-            ('referral_entry_flags', FlagField(size=2, flag_type=DFSReferralEntryFlags)),
-            ('proximity', IntField(size=4)),
-            ('time_to_live', IntField(size=4)),
-            ('dfs_path_offset', IntField(size=2)),
-            ('dfs_alternate_path_offset', IntField(size=2)),
-            ('network_address_offset', IntField(size=2)),
-        ])
+        self.fields = OrderedDict(
+            [
+                ("version_number", IntField(size=2, default=2)),
+                ("size", IntField(size=2)),
+                ("server_type", FlagField(size=2, flag_type=DFSServerTypes)),
+                ("referral_entry_flags", FlagField(size=2, flag_type=DFSReferralEntryFlags)),
+                ("proximity", IntField(size=4)),
+                ("time_to_live", IntField(size=4)),
+                ("dfs_path_offset", IntField(size=2)),
+                ("dfs_alternate_path_offset", IntField(size=2)),
+                ("network_address_offset", IntField(size=2)),
+            ]
+        )
         self.dfs_path = None
         self.dfs_alternate_path = None
         self.network_address = None
         super(DFSReferralEntryV2, self).__init__()
 
     def process_string_buffer(self, buffer, entry_offset):
-        buffer_fields = ['dfs_path', 'dfs_alternate_path', 'network_address']
+        buffer_fields = ["dfs_path", "dfs_alternate_path", "network_address"]
 
         for field_name in buffer_fields:
-            field_offset = self['%s_offset' % field_name].get_value()
+            field_offset = self["%s_offset" % field_name].get_value()
             if field_offset == 0:
                 continue
 
-            field = TextField(null_terminated=True, encoding='utf-16-le')
-            field.unpack(buffer[field_offset - entry_offset:])
+            field = TextField(null_terminated=True, encoding="utf-16-le")
+            field.unpack(buffer[field_offset - entry_offset :])
             setattr(self, field_name, field.get_value())
 
 
@@ -375,47 +406,48 @@ class DFSReferralEntryV3(Structure):
     """
 
     def __init__(self):
-        self.fields = OrderedDict([
-            ('version_number', IntField(size=2, default=3)),
-            ('size', IntField(size=2)),
-            ('server_type', FlagField(size=2, flag_type=DFSServerTypes, flag_strict=False)),
-            ('referral_entry_flags', FlagField(size=2, flag_type=DFSReferralEntryFlags, flag_strict=False)),
-            ('time_to_live', IntField(size=4)),
-
-            # The follow fields depend on the ReferralEntryFlags, the actual field names are when the entry is not a
-            # NameListReferral.
-            ('dfs_path_offset', IntField(size=2)),  # SpecialNameOffset
-            ('dfs_alternate_path_offset', IntField(size=2)),  # NumberOfExpandedNames
-            ('network_address_offset', IntField(size=2)),  # ExpandedNameOffset
-            ('service_site_guid', BytesField(size=lambda s: s['size'].get_value() - 18)),  # Padding
-        ])
+        self.fields = OrderedDict(
+            [
+                ("version_number", IntField(size=2, default=3)),
+                ("size", IntField(size=2)),
+                ("server_type", FlagField(size=2, flag_type=DFSServerTypes, flag_strict=False)),
+                ("referral_entry_flags", FlagField(size=2, flag_type=DFSReferralEntryFlags, flag_strict=False)),
+                ("time_to_live", IntField(size=4)),
+                # The follow fields depend on the ReferralEntryFlags, the actual field names are when the entry is not a
+                # NameListReferral.
+                ("dfs_path_offset", IntField(size=2)),  # SpecialNameOffset
+                ("dfs_alternate_path_offset", IntField(size=2)),  # NumberOfExpandedNames
+                ("network_address_offset", IntField(size=2)),  # ExpandedNameOffset
+                ("service_site_guid", BytesField(size=lambda s: s["size"].get_value() - 18)),  # Padding
+            ]
+        )
         self.dfs_path = None
         self.dfs_alternate_path = None
         self.network_address = None
         super(DFSReferralEntryV3, self).__init__()
 
     def process_string_buffer(self, buffer, entry_offset):
-        is_name_list = self['referral_entry_flags'].has_flag(DFSReferralEntryFlags.NAME_LIST_REFERRAL)
-        buffer_fields = ['dfs_path', 'network_address']
+        is_name_list = self["referral_entry_flags"].has_flag(DFSReferralEntryFlags.NAME_LIST_REFERRAL)
+        buffer_fields = ["dfs_path", "network_address"]
         if not is_name_list:
-            buffer_fields.insert(1, 'dfs_alternate_path')
+            buffer_fields.insert(1, "dfs_alternate_path")
 
         for field_name in buffer_fields:
-            field_offset = self['%s_offset' % field_name].get_value()
+            field_offset = self["%s_offset" % field_name].get_value()
             if field_offset == 0:
                 continue
 
             string_offset = field_offset - entry_offset
-            if is_name_list and field_name == 'network_address':
+            if is_name_list and field_name == "network_address":
                 value = []
-                for _ in range(self['dfs_alternate_path_offset'].get_value()):
-                    field = TextField(null_terminated=True, encoding='utf-16-le')
+                for _ in range(self["dfs_alternate_path_offset"].get_value()):
+                    field = TextField(null_terminated=True, encoding="utf-16-le")
                     field.unpack(buffer[string_offset:])
                     value.append(field.get_value())
                     string_offset += len(field)
 
             else:
-                field = TextField(null_terminated=True, encoding='utf-16-le')
+                field = TextField(null_terminated=True, encoding="utf-16-le")
                 field.unpack(buffer[string_offset:])
                 value = field.get_value()
 
